@@ -26,6 +26,7 @@ __all__ = [
     'ComplexTerm',
     'FactBase',
     'SymbolPredicateUnifier',
+    'Unifier',
     'ContextBuilder',
     'TypeCastSignature',
     'Query',
@@ -66,12 +67,19 @@ __all__ = [
     'noclingo_to_clingo',
     'set_symbol_mode',
     'get_symbol_mode',
-    'symbols',
+    'get_Infimum',
+    'get_Supremum',
+    'Symbol',
+    'Function',
+    'Tuple_',
+    'String',
+    'Number',
     'control_add_facts',
     'symbolic_atoms_to_facts',
     'parse_fact_string',
     'parse_fact_files',
     "ConstantStr",
+    "StrictBool",
     "HeadList",
     "HeadListReversed",
     "TailList",
